gfx = love.graphics

local alpha = 255 -- Can be used to change the construction line/points alphas
local playing = false
local paused = false
local points = {{}}
local line_points = {}
local t = 0
local time_run = 0
local time_to_run = 2 --in seconds


function love.load()
	
end

function love.update(dt)
	if playing and t<1 then
		if not paused then
			t = t + (dt/time_to_run)
			time_run = time_run+dt
			if t>1 or time_run > time_to_run then
				t = 1
				time_run = time_to_run
			end
			--Move points to next point
			for i=2,#points do
				for n=1,#points[i] do
					points[i][n] = lerp(points[i-1][n],points[i-1][n+2],t)
				end
			end
			--Draw the movement segment to the line_points array
			addPoint(line_points,points[#points][1],points[#points][2])
		end
	end
end

function love.mousepressed(x,y,b)
	if not playing then
		if b=="l" then
			--Add a static point
			addPoint(points[1],x,y)
		elseif b=="r" then
			removePoint(points[1])
		elseif b=="m" then
			points[1]={}
		elseif b=="wu" then
			time_to_run = time_to_run+1
		elseif b=="wd" and time_to_run>1 then
			time_to_run = time_to_run-1
		end
	end
end

function love.keypressed(k,ir)
	if k=="v" then if alpha==255 then alpha = 0 else alpha = 255 end end
	if k=="p" then if playing then paused=not paused end end
	if k==" " then
		if playing then
			playing = false
			paused = false
			-- Remove all linear_points
			for i=2,#points,1 do
				points[i] = nil
			end
			t = 0
			time_run = 0
			line_points = {}
		else
			playing = true
			paused = false
			t = 0
			time_run = 0
			line_points = {}
			--Construct the required length of points tables
			--We need 1 less for every point in points[1]
			local needed = (#points[1]/2)-1
			for i=1,needed do
				points[#points+1] = {}
			end
			--
			for i=1,#points do
				for n=1,#points[i]-2,2 do
					addPoint(points[i+1],points[i][n],points[i][n+1])
				end
			end
			-- Add the starting point to the line_points table
			addPoint(line_points,points[#points][1],points[#points][2])
			--
		end
	end
end

local i,j,n --Looping variable
function love.draw()
	gfx.setLineWidth(1)
	gfx.setLineStyle("rough")
	for i=1,#points do
		-- Draw lines
		gfx.setColor(25,25,25,alpha)
		if(#points[i]>3) then
			gfx.line(points[i])
		end
		-- Draw points
		for n=1,#points[i],2 do
			if i==#points and #points~=1 then gfx.setColor(255,163,70,alpha) else gfx.setColor(255,40,40,alpha) end
			gfx.circle("fill",points[i][n],points[i][n+1],4)
		end
	end
	-- Draw produced line curve
	gfx.setLineWidth(3)
	gfx.setLineStyle("smooth")
	gfx.setColor(255,163,70,255)
	if #line_points>3 then
		gfx.line(line_points)
	end
	-- Labels
	gfx.setColor(255,255,255,255)
	gfx.print("Static Points: " ..#points[1]/2,5,5)
	gfx.print("Alpha: " ..alpha,5,15)
	gfx.print("t: " ..t,5,25)
	gfx.print("tr: " ..time_run,5,35)
	gfx.print("table-depth: " ..#points,5,45)
	gfx.print("time to run: " ..time_to_run,5,55)
	gfx.print("fps: " ..love.timer.getFPS(),5,65)
	if playing then
		gfx.setColor(255,0,0,255)
		gfx.print("PLAYING",400,5)
	end
end

--

function addPoint(t,x,y)
	t[#t+1],t[#t+2]=x,y
end

function removePoint(t)
	if #t<2 then return end
	t[#t-1],t[#t]=nil,nil
end

function lerp(a,b,t)
	return a+(b-a)*t
end